// ACP 5.7 ABS(A1) sets AV0
# mach: bfin

.include "testutils.inc"
	start

	r1=0x80 (z);
	A0=0;
	A0.x=r1;
	A0=abs A0;
	_DBG astat;
//r7=astat;
//dbga (r7.h, 0x3);
//dbga (r7.l, 0x0);
	cc = az;
	r7 = cc;
	dbga( r7.l, 0);
	cc = an;
	r7 = cc;
	dbga( r7.l, 0);
	cc = av0;
	r7 = cc;
	dbga( r7.l, 1);
	cc = av0s;
	r7 = cc;
	dbga( r7.l, 1);
	cc = av1;
	r7 = cc;
	dbga( r7.l, 0);
	cc = av1s;
	r7 = cc;
	dbga( r7.l, 0);

	r6=A0.x;
	dbga (r6.l, 0x7f);

	r1=0x80 (z);
	A1=0;
	A1.x=r1;
	A1=abs A1;
	_DBG astat;
//r7=astat;
//dbga (r7.h, 0xf);
//dbga (r7.l, 0x0);
	cc = az;
	r7 = cc;
	dbga( r7.l, 0);
	cc = an;
	r7 = cc;
	dbga( r7.l, 0);
	cc = av0;
	r7 = cc;
	dbga( r7.l, 1);
	cc = av0s;
	r7 = cc;
	dbga( r7.l, 1);
	cc = av1;
	r7 = cc;
	dbga( r7.l, 1);
	cc = av1s;
	r7 = cc;
	dbga( r7.l, 1);

	r6=A1.x;
	dbga (r6.l, 0x7f);

	r7=0;
	astat=r7;
	r1=0x80 (z);
	A1=0;
	A1.x=r1;
	A0 = abs A1;
	_DBG astat;
//r7=astat;
//dbga (r7.h, 0x3);
//dbga (r7.l, 0x0);
	cc = az;
	r7 = cc;
	dbga( r7.l, 0);
	cc = an;
	r7 = cc;
	dbga( r7.l, 0);
	cc = av0;
	r7 = cc;
	dbga( r7.l, 1);
	cc = av0s;
	r7 = cc;
	dbga( r7.l, 1);
	cc = av1;
	r7 = cc;
	dbga( r7.l, 0);
	cc = av1s;
	r7 = cc;
	dbga( r7.l, 0);

	r6=A0.x;
	dbga (r6.l, 0x7f);

	r7=0;
	astat=r7;
	r1=0x80 (z);
	A0=0;
	A0.x=r1;
	A1 = abs A0;
	_DBG astat;
//r7=astat;
//dbga (r7.h, 0xc);
//dbga (r7.l, 0x0);
	cc = az;
	r3 = cc;
	dbga( r3.l, 0);
	cc = an;
	r3 = cc;
	dbga( r3.l, 0);
	cc = av0;
	r3 = cc;
	dbga( r3.l, 0);
	cc = av0s;
	r3 = cc;
	dbga( r3.l, 0);
	cc = av1;
	r3 = cc;
	dbga( r3.l, 1);
	cc = av1s;
	r3 = cc;
	dbga( r3.l, 1);

	r6=A1.x;
	dbga (r6.l, 0x7f);

	r7=0;
	astat=r7;
	r1=0x80 (z);
	A1=0;
	A1.x=r1;
	A0.x=r6;
	_DBG A1;
	_DBG A0;
	A1=abs A1, A0=abs A0;
	_DBG ASTAT;
//r7=astat;
//dbga (r7.h, 0xc);
//dbga (r7.l, 0x0);
	cc = az;
	r4 = cc;
	dbga( r4.l, 0);
	cc = an;
	r4 = cc;
	dbga( r4.l, 0);
	cc = av0;
	r4 = cc;
	dbga( r4.l, 0);
	cc = av0s;
	r4 = cc;
	dbga( r4.l, 0);
	cc = av1;
	r4 = cc;
	dbga( r4.l, 1);
	cc = av1s;
	r4 = cc;
	dbga( r4.l, 1);

	r7=0;
	astat=r7;
	r1=0x80 (z);
	A1=0;
	A1.x=r1;
	A0 = A1;
	A1=abs A1, A0=abs A0;
	_DBG ASTAT;
//r7=astat;
//dbga (r7.h, 0xf);
//dbga (r7.l, 0x0);
	cc = az;
	r7 = cc;
	dbga( r7.l, 0);
	cc = an;
	r7 = cc;
	dbga( r7.l, 0);
	cc = av0;
	r7 = cc;
	dbga( r7.l, 1);
	cc = av0s;
	r7 = cc;
	dbga( r7.l, 1);
	cc = av1;
	r7 = cc;
	dbga( r7.l, 1);
	cc = av1s;
	r7 = cc;
	dbga( r7.l, 1);

// ACP 5.8 ABS sometimes sets AN

	r7=0;
	astat=r7;
	r0=1;
	r1=abs r0;
	_DBG r0;
	_DBG r1;
	_DBG astat;
//r7=astat;
//dbga (r7.h, 0x0);
//dbga (r7.l, 0x0);
	cc = az;
	r7 = cc;
	dbga( r7.l, 0);
	cc = an;
	r7 = cc;
	dbga( r7.l, 0);
	cc = av0;
	r7 = cc;
	dbga( r7.l, 0);
	cc = av0s;
	r7 = cc;
	dbga( r7.l, 0);
	cc = av1;
	r7 = cc;
	dbga( r7.l, 0);
	cc = av1s;
	r7 = cc;
	dbga( r7.l, 0);

	pass;
