/* Main header file for Cpu tools GENerated simulators.
   Copyright (C) 1998-2017 Free Software Foundation, Inc.
   Contributed by Cygnus Support.

This file is part of GDB, the GNU debugger.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

/* This file must be included after sim-base.h.  */

#ifndef CGEN_SIM_H
#define CGEN_SIM_H

#include "cgen-defs.h"
#include "cgen-scache.h"
#include "cgen-fpu.h"
#include "cgen-par.h"
#include "cgen-cpu.h"
#include "cgen-trace.h"

/* This is a machine generated file.  */
#include "cpuall.h"

#endif /* CGEN_SIM_H */
